gk_initConf $argv

set LockingDir $gk_library/concurrency/locking
set DistribDir $gk_library/concurrency/distrib-var
source $LockingDir/main/token-net-layer.tcl
source $LockingDir/main/semi-opt-lock.tcl
source $DistribDir/main/distrib.tcl
source $LockingDir/misc/cursor.tcl

# 
# Each element of PiecesList has two elements.  The first is the
# coordinates on the screen at which the piece belongs.  The second is
# a list of four X-Bitmap files, the first being the correctly
# oriented piece and the rest being the same piece rotated 90, 180,
# and 270 degrees.
# 
source calvin.jig


trace variable NewDistribVar w HandleNewVar
proc HandleNewVar args {
    global NewDistribVar
    set Piece [IndexName $NewDistribVar]
#    puts "HandleNewVar: NewDistribVar = $NewDistribVar, Piece = $Piece"

    .c create bitmap 0 0 -tag $Piece -anchor nw -foreground black -background white
    .c bind $Piece <ButtonPress-1> "GrabPiece $Piece"

    ShowLockWithCursor Pieces($Piece) .c

    global Pieces
    trace variable Pieces($Piece) w UpdatePiecePic
#    puts "HandleNewVar: Done"
}



proc UpdatePiecePic {Array Piece op} {
#    puts "UpdatePiecePic: Moving piece $Piece to [PieceX $Piece] [PieceY $Piece]."
    .c coords $Piece [PieceX $Piece] [PieceY $Piece]
    if {[file exists [PieceBitmap $Piece]] == 0} {
	update
	ModalInfoBox "Unable to find bitmap file [PieceBitmap $Piece]."
	exit 1
    }
    .c itemconfigure $Piece -bitmap @[PieceBitmap $Piece]
}


proc JigsawStateChange {Array Piece op} {
#    puts "JigsawStateChange:"
    global LockStates
    set State $LockStates($Piece)
    set Piece [IndexName $Piece]
#    puts "JigsawStateChange: to $State"
    if {$State == "Tentative"} {
	.c bind $Piece <ButtonPress-1> ""
	.c bind $Piece <B1-Motion> "MovePiece $Piece %x %y"
	.c bind $Piece <Any-ButtonPress-2> "RotatePiece $Piece"
    }

    if {$State == "Approved"} {
	.c bind $Piece <ButtonRelease-1> "DropPiece $Piece"
    }

    if {$State == "Denied" || $State == "Released"} {
	.c bind $Piece <ButtonPress-1> "GrabPiece $Piece"
	.c bind $Piece <B1-Motion> ""
	.c bind $Piece <ButtonRelease-1> ""
	.c bind $Piece <Any-ButtonPress-2> ""
	global LockStates
	trace vdelete LockStates(Pieces($Piece)) w JigsawStateChange
    }
}


proc GrabPiece Piece {
#    puts "GrabPiece: $Piece"
    .c raise $Piece

    global LockStates
    trace variable LockStates(Pieces($Piece)) w JigsawStateChange
    GrabVar Pieces($Piece)
#    puts "GrabPiece - done"
}


proc MovePiece {Piece X Y} {
#    puts "MovePiece: $Piece to $X $Y"
    global Pieces
    keylset Pieces($Piece) Location "$X $Y"
}

proc DropPiece Piece {
#    puts "DropPiece: $Piece"
    global Pieces
    if {[Distance [keylget Pieces($Piece) Location] [keylget Pieces($Piece) Dest]] < 30 && [PieceRot $Piece] == 0} {
#	puts "DropPiece: Jumping piece into place."
        puts -nonewline stdout ""; flush stdout
	MovePiece $Piece [PieceDestX $Piece] [PieceDestY $Piece]
        .c lower $Piece
    }
    ReleaseVar Pieces($Piece)
}


proc RotatePiece Piece {
#    puts "RotatePiece: "
    global Pieces
    set Rot [keylget Pieces($Piece) Rot]
    incr Rot
    if {$Rot > 3} {set Rot 0}
    keylset Pieces($Piece) Rot $Rot
}


proc PieceX Piece {
    global Pieces
    return [lindex [keylget Pieces($Piece) Location] 0]
}


proc PieceY Piece {
    global Pieces
    return [lindex [keylget Pieces($Piece) Location] 1]
}


proc PieceDestX Piece {
    global Pieces
    return [lindex [keylget Pieces($Piece) Dest] 0]
}


proc PieceDestY Piece {
    global Pieces
    return [lindex [keylget Pieces($Piece) Dest] 1]
}


proc PieceBitmap Piece {
    global Pieces
    set Bitmaps [keylget Pieces($Piece) Bitmaps]
    set Rot [keylget Pieces($Piece) Rot]
    set Ans [lindex $Bitmaps $Rot]
    return $Ans
}


proc PieceRot Piece {
    global Pieces
    return [keylget Pieces($Piece) Rot]
}


proc Distance {P1 P2} {
    set Ans [expr [Abs [expr [lindex $P1 0]-[lindex $P2 0]]]+[Abs [expr [lindex $P1 1]-[lindex $P2 1]]]]
    return $Ans
}

proc Abs X {
    if {$X < 0} {return [expr -$X]} {return $X}
}

proc IndexName VarName {
    return [lindex [split $VarName "()"] 1]
}


proc JigsawHelp {} {
    if {[winfo exists .jigsawhelp] == 1} { return "" }
    toplevel .jigsawhelp
    label .jigsawhelp.title -text "Jigsaw Puzzle" -font -Adobe-times-bold-r-normal--*-240*
    label .jigsawhelp.author -text "By David Marwood"
    message .jigsawhelp.mess -text "Try to assemble the jigsaw puzzle.  Click and drag with the left button to move pieces and click the middle button to rotate pieces while dragging them.\nTry to grab a puzzle piece someone else holds.  Try this over a slow network.\nHave fun!" -aspect 300
    button .jigsawhelp.ok -text "Close" -command "destroy .jigsawhelp"
    pack append .jigsawhelp .jigsawhelp.title top .jigsawhelp.author top .jigsawhelp.mess top .jigsawhelp.ok top
}



gk_defaultMenu .menu
.menu.help.menu add command -label "Help on Jigsaw Puzzle" -command "JigsawHelp" -underline 0
pack append . .menu {top fillx}

pack append . [canvas .c -width [expr $PicWidth+200] -height [expr $PicHeight+200] -background blue] top
.c create line 100 100 [expr $PicWidth+100] 100
.c create line 100 100 100 [expr $PicHeight+100]
.c create line 100 [expr $PicHeight+100] [expr $PicWidth+100] [expr $PicHeight+100]
.c create line [expr $PicWidth+100] 100 [expr $PicWidth+100] [expr $PicHeight+100]


if {[gk_amOriginator]} {
    update
    toplevel .pleasewait
    wm title .pleasewait Wait!
    message .pleasewait.mess -text "Please wait while the jigsaw puzzle is started."
    pack append .pleasewait .pleasewait.mess top
    update

    foreach Piece $PiecesList {
#	puts "Initing piece $Piece"
	set LocX [expr [random $PicWidth]+100]
	set LocY [expr [random $PicHeight]+100]
	set DestX [expr [lindex [lindex $Piece 0] 0]+100]
	set DestY [expr [lindex [lindex $Piece 0] 1]+100]
    
	set Bitmaps [lindex $Piece 1]
	set Rot [random 4]
	set Name [lindex [split [lindex $Bitmaps 0] "."] 0]
        while {[info exists Pieces($Name)]} {set Name ${Name}-x}
	keylset Pieces($Name) Location "$LocX $LocY"
	keylset Pieces($Name) Dest "$DestX $DestY"
	keylset Pieces($Name) Bitmaps $Bitmaps
	keylset Pieces($Name) Rot $Rot
	DistributeVar Pieces($Name)
        update
    }
    destroy .pleasewait
}


bind .c <Any-Motion> {gk_callMoveCursor %x %y .c}
gk_makeRemoteCursors .c
