gk_initConf $argv

set LockingDir $gk_library/concurrency/locking
set DistribDir $gk_library/concurrency/distrib-var

# Include the network layer.
source $LockingDir/main/simple-net-layer.tcl
# source $LockingDir/main/token-net-layer.tcl

# Include the lock layer.
source $LockingDir/main/semi-opt-lock.tcl
# source $LockingDir/main/non-opt-lock.tcl

# Include the ditributed variables package.
source $DistribDir/main/distrib.tcl

# Include smart cursors.
source $LockingDir/misc/cursor.tcl


# if you have the program "play", you can set Sound to 1.
set Sound 0

proc DrawLine args {
    global LineTag
    global Point1 Point2
    .canvas coord $LineTag [Point1X] [Point1Y] [Point2X] [Point2Y]
}

proc GrabLine {X Y} {
#    puts "GrabLine: at $X, $Y."
    if {[expr [abs [expr $X-[Point1X]]]+[abs [expr $Y-[Point1Y]]]] < 
	[expr [abs [expr $X-[Point2X]]]+[abs [expr $Y-[Point2Y]]]]} {
	    GrabPoint Point1
	} else {
	    GrabPoint Point2
	}
    DrawLine
}


proc GrabPoint PointName {
    puts "GrabPoint: grabbing $PointName."
    global LockStates
    trace variable LockStates($PointName) w LockStateChange
    bind .canvas <1> {} 
    GrabVar $PointName
}


proc DropPoint PointName {
    ReleaseVar $PointName

    global LockStates
    trace vdelete LockStates($PointName) w LockStateChange
}


proc LockStateChange {Array LockName op} {
    global LockStates
    set LockState $LockStates($LockName)

    puts "LockStateChange: Lock $LockName became $LockState."

    global LineTag
    global LockLayerName
    global Sound
    if {$LockState == "Tentative" || ($LockLayerName == "non-optimistic" && $LockState == "Approved")} {
	if {$LockState == "Tentative" && $Sound == 1} {exec play hmmm.au &}
	bind .canvas <B1-Motion> "set $LockName {%x %y}"
	.canvas itemconfigure $LineTag -fill red
	# set cursor to question mark
    }

    if {$LockState == "Approved"} {
	if {$Sound == 1} {exec play ok.au &}
#	puts "LockStateChange: Approved"
	bind .canvas <ButtonRelease-1> "DropPoint $LockName"
	.canvas itemconfigure $LineTag -fill slateBlue
	# set cursor to normal arrow
    }

    if {$LockState == "Released"} {
	if {$Sound == 1} {exec play done.au &}
#	puts "LockStateChange: Released"
	bind .canvas <B1-Motion> {}
	bind .canvas <ButtonRelease-1> {}
	bind .canvas <1> {GrabLine %x %y}
	.canvas itemconfigure $LineTag -fill black
	# set cursor to arrow
    }

    if {$LockState == "Denied"} {
	if {$Sound == 1} {exec play doh.au &}
#	puts "LockStateChange: Denied"
	bind .canvas <B1-Motion> {}
	bind .canvas <ButtonRelease-1> {}
	bind .canvas <1> {GrabLine %x %y}
	trace vdelete LockStates($LockName) w LockStateChange
	.canvas itemconfigure $LineTag -fill black
	# set cursor to arrow
    }
}


proc Point1X {} {global Point1; return [lindex $Point1 0]}
proc Point1Y {} {global Point1; return [lindex $Point1 1]}
proc Point2X {} {global Point2; return [lindex $Point2 0]}
proc Point2Y {} {global Point2; return [lindex $Point2 1]}
proc abs A {if {$A<0} {return [expr -$A]} else {return $A}}


set Point1 "5 5"
set Point2 "20 20"

if {[gk_amOriginator]} {
    DistributeVar Point1
    DistributeVar Point2
}

ShowLockWithCursor Point1 .canvas
ShowLockWithCursor Point2 .canvas

trace variable Point1 w DrawLine
trace variable Point2 w DrawLine

canvas .canvas
pack append . .canvas bottom
set LineTag [.canvas create line [Point1X] [Point1Y] [Point2X] [Point2Y]]

bind .canvas <1> {GrabLine %x %y}


# 
# Create the standard menu and add a help selection.
# 
proc SingleLineHelp {} {
    if {[winfo exists .help] == 1} { return "" }
    toplevel .help
    label .help.title -text "Single Locked Line" -font -Adobe-times-bold-r-normal--*-240*
    label .help.author -text "By David Marwood"
    message .help.mess -text "This is a demonstration of the various locking schemes available in the GroupKit Concurrency Control Package.  To try a different locking scheme, edit the single-line-lock.tcl file to source a different lock or network layer." -aspect 300
    button .help.ok -text "Close" -command "destroy .help"
    pack append .help .help.title top .help.author top .help.mess top .help.ok top
}

gk_defaultMenu .menu
.menu.help.menu add command -label "Help on Single Locked Line" -command "SingleLineHelp" -underline 0
pack append . .menu {top fillx}
