# ----------------------------------------------------------------
# Initialize GroupKit and a simple non-optimistic lock.
# ----------------------------------------------------------------

gk_initConf $argv

set LockingDir $gk_library/concurrency/locking
source $LockingDir/main/simple-net-layer.tcl
source $LockingDir/main/non-opt-lock.tcl


# Only one member of the conference should create the lock.
if {[gk_amOriginator]} {
    CreateLock ControlsLock
}


# ----------------------------------------------------------------
# Main procedures for concurrency control
# ----------------------------------------------------------------

# 
# Called when the MakeChanges button is pressed.
# 
proc MakeChanges {} {
    .controlButton config -state disabled
    GrabLock ControlsLock
}


# 
# Makes changes to interface as the lock state changes.
# 
trace variable LockStates(ControlsLock) w ControlsLockStateChange
proc ControlsLockStateChange args {
    global LockStates

    set State $LockStates(ControlsLock)

    if {$State == "Approved"} {
	EnableControls
	.controlButton config -text "Press Here when Done" -command \
	    "ReleaseLock ControlsLock" -state normal
    }

    if {$State == "Denied"} {
	.controlButton config -state normal
    }

    if {$State == "Released"} {
	DisableControls
	.controlButton config -text "Press Here to Make Changes" -command \
	    {catch {MakeChanges}} -state normal
    }
}


# ----------------------------------------------------------------
# Utilities
# ----------------------------------------------------------------

# 
# Enable and disable the controls.
# 
proc EnableControls {} {
    .coolant config -state normal
    .power config -state normal
    bind .coolant <ButtonPress-1> {gk_toOthers SetScale .coolant \
	[.coolant get]}
    bind .power <ButtonPress-1> {gk_toOthers SetScale .power \
	[.power get]}
    bind .coolant <B1-Motion> {gk_toOthers SetScale .coolant \
	[.coolant get]}
    bind .power <B1-Motion> {gk_toOthers SetScale .power [.power get]}
}


proc DisableControls {} {
    .coolant config -state disabled
    .power config -state disabled
    bind .coolant <ButtonPress-1> {}
    bind .power <ButtonPress-1> {}
    bind .coolant <B1-Motion> {}
    bind .power <B1-Motion> {}
}


# 
# Set transmitted changes.
# 
proc SetScale {Scale Val} {
    $Scale config -state normal
    $Scale set $Val
    $Scale config -state disabled
}


# ----------------------------------------------------------------
# Screen Initialization
# ----------------------------------------------------------------

# Create and pack the button and scales.
button .controlButton -text "Press Here to Make Changes" \
                      -command {catch {MakeChanges}} -pady 0.1c
scale .coolant -from 1 -to 10 -orient horizontal \
               -label "Coolant Flow Rate" -length 10c
scale .power -from 1 -to 10 -orient horizontal \
             -label "Power Output" -length 10c
pack append . .controlButton {top fillx} \
              .coolant {top fill expand} \
              .power {top fill expand}

# 
# Initialize the control to disabled, since the lock is initialized to
# Released.
# 
DisableControls

wm maxsize . 1000 1000
