#
#  This file is part of GroupKit
#
#  (c) Copyright 1992 Department of Computer Science, University of
#      Calgary, Calgary, Alberta, Canada.  All rights reserved.
#
#  Permission to use, copy, modify, and distribute this software and its
#  documentation for any purpose and without fee is hereby granted, provided
#  that the above copyright notice appears in all copies.  The University
#  of Calgary makes no representations about the suitability of this
#  software for any purpose.  It is provided "as is" without express or
#  implied warranty.
#

# Written by David Marwood

# source ~grouplab/repository-co/handy_procs/simple/tcl-path.tcl
# source [SearchTCLPath tclgk-lib.tcl]

# ----------------------------------------------------------------
# Algorithm
# ----------------------------------------------------------------

# NetworkRequest is called and transmits a RemoteRequest to all
# others.  Each other must then send back a RemoteResponse saying
# whether or not the lock should be approved.  If all others say the
# lock should be approved then it is approved, otherwise it is denied.


# ----------------------------------------------------------------
# External Data
# ----------------------------------------------------------------
set NetworkLayerName "simple"

# ----------------------------------------------------------------
# Internal Data structures
# ----------------------------------------------------------------

# NeededReponses: Three arrays are used in concert.  When a lock
# LockName is requested, NeededResponse(LockName) is set to the number
# of others and RemoteRequest is sent to all the others.

# YesResponses: When a yes responses for lock LockName is received by
# RemoteResponses, YesResponses(LockName) is incremented.

# Responses: When a responses for lock LockName is received,
# Responses(LockName) is incremented.


# ----------------------------------------------------------------
# Procedures executed by lock requestor.
# ----------------------------------------------------------------

proc NetworkCreateLock LockName {}

# 
# Sets Responses and YesResponses to 0, NeededResponses($LockName) to
# number of others.  Sends RemoteRequest to all others.  Returns
# nothing.
# 
proc NetworkRequest LockName {
    global Responses
    set Responses($LockName) 0
    
    global YesResponses
    set YesResponses($LockName) 0
    
    global NeededResponses
    global _gk_remoteUsers
    set NeededResponses($LockName) [llength $_gk_remoteUsers]

    global _gk_remoteUsers
    global _gk_localUser
#    after 1000
#    sleep 1
#    puts "NetworkRequest: Transmitting to other users."
    gk_toOthers RemoteRequest $LockName [keylget _gk_localUser usernum]

    CheckDone $LockName
#    puts "NetworkRequest: Done."
}


# 
# Called by others.  LockName is the name of a lock, and YesNo is Yes
# iff the caller does want the lock.  Calls CheckDone.
# 
proc RemoteResponse {LockName YesNo} {
#    puts "RemoteResponse: Got a '$YesNo' for lock $LockName."
    global Responses
    incr Responses($LockName)

    global YesResponses
    if {$YesNo == "Yes"} {
	incr YesResponses($LockName)
    }
    CheckDone $LockName
}


# 
# If enough (NeededReponses($LockName)) have been received, then
# NetworkResponse is called.  NetworkResponse is passed a Yes iff all
# responses were Yes.
# 
proc CheckDone LockName {
    global Responses
    global NeededResponses
    global YesResponses
    
    if {$Responses($LockName) == $NeededResponses($LockName)} {
	if {$Responses($LockName) == $YesResponses($LockName)} {
	    NetworkResponse $LockName Yes
	} else {
	    NetworkResponse $LockName No
	}
	unset NeededResponses($LockName)
	unset Responses($LockName)
    }
}


# ----------------------------------------------------------------
# Procedures executed by remote sites.
# ----------------------------------------------------------------

# 
# Called by a remote host wanting to hold the lock.  Transmits
# RemoteResponse; passes a Yes iff the lock is not desired or held.
# 
proc RemoteRequest {LockName Usernum} {
#    puts "RemoteRequest: for $LockName from $Usernum."
    global _gk_remoteUsers
    global HeldLocks
    global Responses
#    after 1000 
    if {[lsearch $HeldLocks $LockName] != -1 ||
	[info exists Responses($LockName)] == 1} {
	gk_toUserNum $Usernum RemoteResponse $LockName No
    } else {
	gk_toUserNum $Usernum RemoteResponse $LockName Yes
    }
}
