#
# widgets for GK conferences to use
#

#############################################################
# This is a generic procedure used to configure all children
# of a certain window.  Call as follows:
# gk_winConfigure window commands
# where commands is, for example, -bg yellow -fg pink
#
proc gk_winConfigure {args} {                 
    set window [lindex $args 0]
    set len [llength $args]

    foreach x [winfo children $window] {
        eval "gk_winConfigure $x [join [lreplace $args 0 0]]"
    }
    
    catch {
        for {set count 1} {$count < $len} {incr count 1} {
            $window configure [lindex $args $count] \
              [lindex $args [incr count 1]]
        }
    }
}
#############################################################
#  This creates the default GroupKit menubar with three pulldown
#  menus---File, Collaborators, and Help.  It is called
#  with one parameter---the menu bar.
#  This menu may be re-configured to add save, open etc. options.
#
proc gk_defaultMenu menuBar {
    frame $menuBar -borderwidth 2 -relief raised

    menubutton $menuBar.file -menu $menuBar.file.menu -text File -underline 0
    menu $menuBar.file.menu
    $menuBar.file.menu add command -label "Quit" -command "_gk_properQuit" -underline 0

    menubutton $menuBar.help -menu $menuBar.help.menu -text Help -underline 0
    menu $menuBar.help.menu
    $menuBar.help.menu add command -label "About GroupKit" -command "gk_about"

    menubutton $menuBar.collaborators -menu $menuBar.collaborators.menu -text Collaboration -underline 0
    menu $menuBar.collaborators.menu
    $menuBar.collaborators.menu add check -label "Show Users" -command "_gk_showUsers" -variable gkUsersWindowShowing -underline 5

    pack append $menuBar $menuBar.file left $menuBar.help right $menuBar.collaborators right
}
#############################################################

#############################################################
#  The default GroupKit registrar client menu bar.  It is the same as above
#  except that it does not have the 'Collaborators' menu.
#
proc gk_defaultRCMenu menuBar {
    frame $menuBar -borderwidth 2 -relief raised

    menubutton $menuBar.file -menu $menuBar.file.menu -text File -underline 0
    menu $menuBar.file.menu
    $menuBar.file.menu add command -label "Quit" -command "_gk_properQuit" -underline 0

    menubutton $menuBar.help -menu $menuBar.help.menu -text Help -underline 0
    menu $menuBar.help.menu
    $menuBar.help.menu add command -label "About GroupKit" -command "gk_about"

    pack append $menuBar $menuBar.file left $menuBar.help right
}
#############################################################

proc _gk_showUsers {} {  global gkUsersWindowShowing _gk_localUser _gk_remoteUsers
    if {$gkUsersWindowShowing == 1} {
	if {[winfo exists .gkUsers] == 0} {
	    # creating for the first time
	    toplevel .gkUsers
	    wm protocol .gkUsers WM_DELETE_WINDOW "set gkUsersWindowShowing 0; wm withdraw .gkUsers"
	    label .gkUsers.lbl -text [concat "Others in" [keylget _gk_localUser confname]] -font -Adobe-times-bold-r-normal--*-180*
	    pack append .gkUsers .gkUsers.lbl top
	    foreach i $_gk_remoteUsers {
		set unum [keylget i usernum]
		label .gkUsers.l$unum -text [keylget i username]
		pack append .gkUsers .gkUsers.l$unum top
	    }
	} else {
	    wm deiconify .gkUsers
	}
    } else {
	if {[winfo exists .gkUsers] == 1} {
	    wm withdraw .gkUsers
	}
    }
}


trace variable gk_deletedUser w _gk_removeUserFromWindow
trace variable gk_newUser w _gk_addUserToWindow

proc _gk_removeUserFromWindow args { global gk_deletedUser
    set unum [keylget gk_deletedUser usernum]
    catch {pack unpack .gkUsers.l$unum}
    return ""
}

proc _gk_addUserToWindow args { global gk_newUser
    if {[winfo exists .gkUsers] == 1} {
	set unum [keylget gk_newUser usernum]
	label .gkUsers.l$unum -text [keylget gk_newUser username]
	pack append .gkUsers .gkUsers.l$unum top
    }
    return ""
}

proc gk_about {} {  global gk_library
    if {[winfo exists .about] == 1} { return "" }
    toplevel .about 
    label .about.icon -bitmap @$gk_library/library/bitmaps/dont_panic.xbm -fg hotpink
    label .about.title -text "GroupKit" -font -Adobe-times-bold-r-normal--*-240*
    message .about.version -text "Version 1.1 \n Aug 20, 1993" -width 200
    message .about.authors -text "By Mark Roseman with help from Saul Greenberg, David Marwood and Salaam Yitbarek"
    message .about.contact -text "For more info, contact Saul Greenberg <saul@cpsc.ucalgary.ca>" -width 300
    button .about.ok -text "Okay" -command "destroy .about"
    pack append .about .about.icon left .about.title top .about.version top .about.authors top .about.contact top .about.ok bottom
}

##################################################
# GENERIC SCROLLBAR STUFF COMING UP
#
##################################################

# 1. Call using following format:
#    groupScroll _theScrollFrame_ -command "_yourScrolledObject_ yview"
# 2. Then pack _theScrollFrame_
# 3. Put a '-yscrollcommand "_yourScrollProc_"' in _yourScrolledObject_
#    declaration.
# 4. groupScroll provides global scrollid
#
# myColour can be set in the .tclgkrc file.  It is the user's colour.
#

##################################################
# _gk_checkMouse is the callback for mouse B1 press on the remote
# scrollbars.  It finds which bars the mouse is on and then finds
# the users that these bars represent.
# It then calls _gk_displayNames to display those users.
#
proc _gk_checkMouse {x y X Y remote} { global scrollid lastBarNames
    set mouseOverThis [$remote find overlapping $x $y $x $y]
    if {$mouseOverThis == ""} { return }

    set barNames {}
    foreach bar $mouseOverThis {
	foreach id [array names scrollid] {
	    if {$scrollid($id) == $bar} {
		if {$id == "self"} { set barNames \
		  [linsert $barNames 0 [gk_getLocalAttrib username]]
		} else { set barNames \
		  [linsert $barNames 0 [gk_getUserAttrib $id username]]
		}

	    }
	}
    }
    _gk_displayNames $barNames $X $Y
}
##################################################

##################################################

proc _gk_checkRelease {} {
    if {[winfo exists .winBarNames] == 1} {
	wm withdraw .winBarNames }
}
##################################################

##################################################
# This displays the names of the users represented by the remote
# scrollbars on which the mouse is.  It create a window and puts
# the names in as labels.
proc _gk_displayNames {barNames X Y} {
    if {[winfo exists .winBarNames] == 0} {
	toplevel .winBarNames
	wm transient .winBarNames .
	wm title .winBarNames People
    }
    foreach i [winfo children .winBarNames] { destroy $i }
    wm geometry .winBarNames +$X+$Y
    set count 0
    foreach name $barNames {
	label .winBarNames.$count -text $name \
	    -font -Adobe-times-medium-r-normal--*-180* \
	    -background aquamarine
	pack append .winBarNames .winBarNames.$count {top fillx}
	incr count 1
    }
    wm deiconify .winBarNames    
}
##################################################

##################################################
#  If myColour has been set by the user (in the .tclgkrc), noting
#  is done.  Otherwise a colour from _gk_colours is assigned to
#  the user.
proc _gk_setMyColour {} { global myColour _gk_colours 
    if {[info exists myColour] == 0} \
      {set myColour $_gk_colours([expr [gk_getLocalAttrib usernum]%20])}
}
##################################################

##################################################
#  Initializes a group scrollbar.
#  1. Establishes myColour.
#  2. Creates local bar and image of local bar on
#     remote bar.
#  3. Configures local bar to command passed in args[1], args[2].
#
proc gk_groupScroll {args} { global scrollid myColour
    global gk_newUser gk_deletedUser
    set scroll [lindex $args 0]
    frame $scroll
    scrollbar $scroll.local -relief sunken
    canvas $scroll.remote -relief sunken -width 15
    _gk_setMyColour

    set scrollid(self) [$scroll.remote create rectangle 0 0 15 50 -fill $myColour]
    pack append $scroll $scroll.local {right filly} $scroll.remote {right filly}
    $scroll.local configure [lindex $args 1] [lindex $args 2]

    trace variable gk_newUser w "gk_newScroll $scroll"
    trace variable gk_deletedUser w "gk_removeScroll $scroll"

    bind $scroll.remote <1> "
	_gk_checkMouse %x %y %X %Y $scroll.remote
    "
    bind $scroll.remote <B1-ButtonRelease> {
	_gk_checkRelease
    }
}
##################################################

# gk_scroller is called whenever the scrollbars need adjusting.  It calls
# _gk_adjustFake to adjust the local and remote scrollbars.  It then
# does '$scroll.local set ...' to adjust the local real scrollbar.
# If we must update a new user, the _gk_existsNewUser flag will be true.  In this
# case, we do _gk_adjustFake with newCoords---the present coords of our real
# scrollbar.

set _gk_existsNewUser false
proc gk_scroller {scroll args} { global _gk_localUser _gk_existsNewUser newCoords
    set totalUnits [lindex $args 0]
    set windowUnits [lindex  $args 1]
    set firstUnit [lindex $args 2]
    set lastUnit [lindex $args 3]

    if {$_gk_existsNewUser == "false"} {
	_gk_adjustFake self $totalUnits $windowUnits $firstUnit $lastUnit $scroll
    } else {
	foreach i [array names newCoords] {
	    set element $newCoords($i)
	    _gk_adjustFake [lindex $element 4] [lindex $element 0] \
	      [lindex $element 1] [lindex $element 2] [lindex $element 3] $scroll
	    set _gk_existsNewUser false
	}
    }

    gk_toOthers _gk_adjustFake [keylget _gk_localUser usernum] \
      $totalUnits $windowUnits $firstUnit $lastUnit $scroll
    eval "$scroll.local set $totalUnits $windowUnits $firstUnit $lastUnit"
}

# This adjusts the fake scrollbar.
proc _gk_adjustFake {userid totalUnits windowUnits firstUnit lastUnit scroll} \
                { global scrollid errorInfo
    if {$totalUnits > $windowUnits} {
        set top [expr ($firstUnit.0/$totalUnits)*[winfo height $scroll.remote]]
        set bott [expr ($lastUnit.0/$totalUnits)*[winfo height $scroll.remote]]
    } else { set top 0; set bott [winfo height $scroll.remote] }
    $scroll.remote coord $scrollid($userid) 0 $top 15 $bott
}

# This initializes the fake scrollbars of new users.
proc gk_newScroll {scroll args} { global gk_newUser scrollid
    set newUserNum [keylget gk_newUser usernum]
    set localUserNum [gk_getLocalAttrib usernum]
    set scrollid($newUserNum) \
      [$scroll.remote create rectangle 0 0 15 50 \
      -fill [_gk_chooseColour $newUserNum $scroll]]
    if {$newUserNum > $localUserNum} {
	gk_toUserNum $newUserNum gk_receiveCoords [$scroll.local get] $localUserNum
    }
}

# Receive the coordinates of the scrollbar of the user who is updating
# us (in case we arrive in the middle of a conference).
proc gk_receiveCoords {theCoords userid} { global _gk_existsNewUser newCoords scrollid
    set _gk_existsNewUser true
    set newCoords($userid) [lappend theCoords $userid]
}

# Removes an deleted user's bar from our fake scrollbar.
proc gk_removeScroll {scroll args} { global gk_deletedUser scrollid
    $scroll.remote coord $scrollid([keylget gk_deletedUser usernum]) 0 0 0 0
    unset scrollid([keylget gk_deletedUser usernum])
}

# Tells remote user to send us his colour.
proc _gk_chooseColour {tableIndex scroll} { global _gk_colours myColour gk_newUser
    gk_toUserNum [keylget gk_newUser usernum] _gk_setColour [gk_getLocalAttrib usernum] $scroll
    return $_gk_colours([expr $tableIndex%20])
}

# Remote user uses this to send his colour.
proc _gk_setColour {usernum scroll} { global myColour _gk_remoteUsers
    while {$_gk_remoteUsers == ""} { tkwait variable _gk_remoteUsers }
    gk_toUserNum $usernum \
      _gk_receiveColour $myColour [gk_getLocalAttrib usernum] $scroll
}

# Remote user has made us do this procedure so we can receive his
# colour.
proc _gk_receiveColour {colour usernum scroll} { global scrollid
    $scroll.remote itemconfigure $scrollid($usernum) -fill $colour
}

#############################################################
set _gk_colours(0) PapayaWhip
set _gk_colours(1) NavajoWhite
set _gk_colours(2) MintCream
set _gk_colours(3) AliceBlue
set _gk_colours(4) DarkSlateGrey
set _gk_colours(5) NavyBlue
set _gk_colours(6) MediumTurquoise
set _gk_colours(7) LightCyan
set _gk_colours(8) DarkOliveGreen
set _gk_colours(9) chartreuse
set _gk_colours(10) OliveDrab
set _gk_colours(11) LightGoldenrodYellow
set _gk_colours(12) IndianRed
set _gk_colours(13) chocolate
set _gk_colours(14) LightSalmon
set _gk_colours(15) VioletRed
set _gk_colours(16) MediumPurple
set _gk_colours(17) burlywood4
set _gk_colours(18) wheat3
set _gk_colours(19) plum3
#############################################################

##########
#
# CURSOR OVERLAYS COMING UP
#
#

#  Initialize array of remote cursors for canvas c.
proc gk_makeRemoteCursors {c} { global cursors gk_deletedUser 
    set cursors($c,0) ""
    trace variable gk_deletedUser w "gk_removeCursor $c"
    _gk_setMyColour
}

#
# Called when the local cursor moves; tell everyone else about this
#
proc gk_callMoveCursor {x y c} { global _gk_localUser myColour
    gk_toOthers _gk_moveCursor [keylget _gk_localUser usernum] $x $y $c $myColour
}
 
#
# Called when a remote cursor is moved.  Move the bitmap for the remote
# cursor.   
#
proc _gk_moveCursor {who x y c colour} { global cursors gk_library
    if {[info exists cursors($c,$who)] != 1} {
	set temp [$c create text 2c 2c -text [gk_getUserAttrib $who username] \
	    -anchor nw -font -Adobe-times-bold-r-normal--11*]
	$c itemconfig $temp -fill $colour
        set cursors($c,$who) $temp
    }
    $c coords $cursors($c,$who) $x $y   
}    
 
# 
# When a user leaves, get rid of their cursor bitmap on the canvas.
#
proc gk_removeCursor {c args} { global gk_deletedUser cursors
    if {[info exists cursors($c,[keylget gk_deletedUser usernum])]} {
        $c delete $cursors($c,[keylget gk_deletedUser usernum])
        unset cursors($c,[keylget gk_deletedUser usernum])
    }
}
